"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var rand_1 = require("./rand");
var apply_1 = require("./apply");
var mistakes = __importStar(require("./mistakes"));
var VALID_MISTAKE_TYPES = {
    omission: "omit",
    duplication: "duplicate",
    repetition: "repeat",
    substitution: "substitute",
    transposition: "transpose",
    miss: "miss",
    misCased: "toggleCase",
    capitalized: "capitalize"
};
var SAMPLE_MISTAKE_CONFIGURATION = {
    miss: {
        probability: 0.033,
        type: "CHARACTER"
    },
    omission: {
        probability: 0.033,
        type: "CHARACTER"
    },
    duplication: {
        probability: 0.033,
        type: "CHARACTER"
    },
    substitution: {
        probability: 0.033,
        type: "CHARACTER"
    },
    transposition: {
        probability: 0.033,
        type: "CHARACTER"
    },
    misCased: {
        probability: 0.033,
        type: "CHARACTER"
    },
    capitalized: {
        probability: 0.033,
        type: "CHARACTER"
    },
    repetition: {
        probability: 0.1,
        type: "WORD"
    }
};
var IGNORED_CHARACTERS = [" "];
function humanize(string, mistakeConfiguration, options) {
    if (mistakeConfiguration === void 0) { mistakeConfiguration = SAMPLE_MISTAKE_CONFIGURATION; }
    if (options === void 0) { options = {}; }
    rand_1.initRand(options.seed);
    return Object.entries(mistakeConfiguration).reduce(function (transformedString, _a) {
        var mistake = _a[0], mistakeConfiguration = _a[1];
        var applicator = mistakes[VALID_MISTAKE_TYPES[mistake]] ||
            mistakeConfiguration.apply;
        if (!applicator) {
            throw new Error("missing `applicator` for " + mistake + ":" + JSON.stringify(mistakeConfiguration));
        }
        return (transformedString
            // Apply word-level mistakes
            .split(" ")
            .map(function (word) {
            if (mistakeConfiguration.type === "WORD") {
                return apply_1.apply(applicator, word, mistakeConfiguration.probability);
            }
            return word;
        })
            .join(" ")
            // Apply char-level mistakes
            .split("")
            .map(function (char) {
            if (mistakeConfiguration.type === "CHARACTER" &&
                !IGNORED_CHARACTERS.includes(char)) {
                return apply_1.apply(applicator, char, mistakeConfiguration.probability);
            }
            return char;
        })
            .join(""));
    }, string);
}
exports.humanize = humanize;
